--Button presses for character movement
D_NEUTRAL       = 0x0
D_UP            = 0x1
D_DOWN          = 0x2
D_FORWARD       = 0x3
D_FORWARD_UP    = 0x4
D_FORWARD_DOWN  = 0x5
D_BACKWARD      = 0x6
D_BACKWARD_UP   = 0x7
D_BACKWARD_DOWN = 0x8

--Button presses for character attack
B_WP = 0x9
B_SP = 0xa
B_WK = 0xb
B_SK = 0xc
B_S  = 0xd

--Combination button presses
B_WPWK = 0xe
B_SPSK = 0xf
B_DOUBLETAP_FW	= 0x20
B_DOUBLETAP_BW	= 0x21

--State flags
F_PUSHABLE = 0x1
F_PUSHING = 0x2
F_BLOCKLOW = 0x4
F_BLOCKHIGH = 0x8
F_HITLOW = 0x10
F_HITMID = 0x20
F_HITHIGH = 0x40
F_TRIP = 0x80
F_INVINCIBLE = 0x100
F_UNSTUNNABLE = 0x200
F_CROUCH = 0x400
F_THROW = 0x800
F_FLIPPABLE = 0x1000
F_IGNOREPUSHBACK = 0x2000
F_IGNOREDAMAGE = 0x4000
F_ANTIGRAVITY = 0x8000
F_WALKING = 0x10000
F_SLOWTIME = 0x20000


--stored player 1 state
playerone = {}
playerone["pointer"] = 0
playerone["state"] = "standing"
playerone["flags"] = 0
playerone["X"] = 0
playerone["Y"] = 0

--stored player 2 state
playertwo = {}
playertwo["pointer"] = 0
playertwo["state"] = "standing"
playertwo["flags"] = 0
playertwo["X"] = 0
playertwo["Y"] = 0


function testflag(set, flag)
  return set % (2*flag) >= flag
end

function setPlayers(p1point, p2point)
	playerone["pointer"] = p1point
	playertwo["pointer"] = p2point
end

--update stored state for player 1
function updateP1(newstate, newflags, newX, newY)
	playerone["state"] = newstate
	playerone["flags"] = newflags
	playerone["X"] = newX
	playerone["Y"] = newY
end

--update stored state for player 2
function updateP2(newstate, newflags, newX, newY)
	playertwo["state"] = newstate
	playertwo["flags"] = newflags
	playertwo["X"] = newX
	playertwo["Y"] = newY
end

function forceOnce (player, statename, input)
	if player["state"] ~= statename
	then forceInput(input, player["pointer"])
	end
end

--makes player move toward other player
--takes a player array
function forceMovement(player, statename, input)
	if player["state"] ~= statename then
		if player["state"] ~= "neutral" then
			forceInput(D_NEUTRAL, player["pointer"])
		else
			forceInput(input, player["pointer"])
		end
	end
end



function randomAttack(player, opponent)
	if(math.random(0,1) == 0) then
		forceInput(D_DOWN, player["pointer"])
	end
		
	if(math.abs(player["X"] - opponent["X"]) < 330) then
		forceInput(B_WP + math.random(0,2), player["pointer"])
	else
		forceInput(B_SK + math.random(0,2), player["pointer"])
	end
end

function attackLow(player)
	forceInput(D_DOWN, player["pointer"])
	if(math.random(0,2) == 0) then
		forceInput(B_S, player["pointer"])
	else
		forceInput(B_SK, player["pointer"])
	end
end

function attackHigh(player)
	if(math.random(0,2) == 0) then
		forceInput(B_S, player["pointer"])
	else
		forceInput(B_SK, player["pointer"])
	end
end

	
--Feed the AI input strings based on what action we want the AI to perform
--Calls C++ function
function MakeAIMove(p1ai, p2ai, level)
	if(p1ai == true) then
		if level == 2 or level == 4 then
			TurtleAI(playerone, playertwo, level)
		else
			VariableAI(playerone, playertwo, level)
		end
	end
	if(p2ai == true) then
		if level == 2 or level == 4 then
			TurtleAI(playertwo, playerone, level)
		else
			VariableAI(playertwo, playerone, level)
		end
	end
end

function VariableAI(player, opponent, difficulty)
	if math.abs(player["X"] - opponent["X"]) > 600 then
		forceMovement(player, "run", B_DOUBLETAP_FW)
	elseif math.abs(player["X"] - opponent["X"]) > (280 + math.random(0,150)) then
		if 	(testflag(opponent["flags"], F_HITMID) 
			or testflag(opponent["flags"], F_HITHIGH))
		then
			if math.random(0,10) <= difficulty then
				if( math.random(0,10) == 0) then
					if testflag(opponent["flags"], F_CROUCH) then
						attackHigh(player)
					else
						attackLow(player)
					end
				else
					forceOnce(player, "walkback", D_BACKWARD)
				end
			end
		elseif 	testflag(opponent["flags"], F_HITLOW) then
			if (math.random(0,10) <= difficulty) then
				if( math.random(0,10) == 0) then
					attackHigh(player)
				else
					forceOnce(player, "b_crouching", D_BACKWARD_DOWN)
				end
			end
		else
			forceMovement(player, "walk", D_FORWARD)
		end
	else
		if math.random(0,10) <= difficulty then
			randomAttack(player, opponent)
		end
	end
end

function TurtleAI(player, opponent, difficulty)
	dist = math.abs(player["X"] - opponent["X"])
	
	if  (dist > 500) and player["X"] < 950 and player["X"] > -950 then
		forceMovement(player, "bdash", B_DOUBLETAP_BW)
	elseif math.abs(player["X"] - opponent["X"]) > (275 + math.random(0,150)) then
		if math.random(0,10) <= difficulty then
			if testflag(opponent["flags"], F_HITLOW) then
				forceOnce(player, "b_crouching", D_BACKWARD_DOWN)
			else
				forceOnce(player, "walkback", D_BACKWARD)
			end
		else
			if math.random(0,150) <= difficulty then
				randomAttack(player, opponent)
			end
		end
	else
		if math.random(0,10) <= difficulty then
			randomAttack(player, opponent)
		end
	end
end